-- This is simplified version of DarkFader's passthrough code.
-- It's used for the PassMe CPLD passthrough.

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Declarations that are provided for instantiating Xilinx
--  primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity PassMe is
    Port ( 
    		 CLK			: in std_logic;

    		 DSSLOT_CLK 	: in std_logic;
           DSSLOT_ROMCS 	: in std_logic;
           DSSLOT_RESET	: in std_logic;
		 DSSLOT_EEPCS 	: in std_logic;
		 DSSLOT_IRQ 	: out std_logic;
           DSSLOT_IO 	: inout std_logic_vector(7 downto 0);

           DSCART_CLK 	: out std_logic;
           DSCART_ROMCS 	: out std_logic;
           DSCART_RESET 	: out std_logic;
		 DSCART_EEPCS 	: out std_logic;
		 DSCART_IRQ 	: in std_logic;
           DSCART_IO 	: inout std_logic_vector(7 downto 0);

		 LED0		: out std_logic;  	-- power LED
		 LED1		: out std_logic);	-- transfer LED
end PassMe;

architecture Behavioral of PassMe is
	signal is_command		: boolean := true;
	signal cmd_cnt			: natural range 0 to 7 := 0;
	signal data_cnt 		: natural range 0 to 8191 := 0;
	signal patch_en 		: boolean := false;
	signal transfer_cnt 	: natural range 0 to 16383 := 0;

	signal dsslot_clk_buf 	: std_logic;
	signal dsslot_romcs_buf 	: std_logic;
begin
   	patch_en <= (transfer_cnt = 1);

------------------------------------------------------------------------------
-- DS slot/card
------------------------------------------------------------------------------

	dsslot_clk_ibuf : IBUF port map ( I => DSSLOT_CLK, O => dsslot_clk_buf );
	dsslot_clk_obuf : OBUF port map ( I => dsslot_clk_buf, O => DSCART_CLK );

	dsslot_romcs_ibuf : IBUF port map ( I => DSSLOT_ROMCS, O => dsslot_romcs_buf );
	dsslot_romcs_obuf : OBUF port map ( I => dsslot_romcs_buf, O => DSCART_ROMCS );


	-- the folloring lines don't have to be reconfigured
	-- they could have been connected on the PCB, but it's
	-- more flexible to leave them here in case something changes
	DSCART_RESET <= DSSLOT_RESET;
	DSSLOT_IRQ <= DSCART_IRQ;
	DSCART_EEPCS <= DSSLOT_EEPCS;

	-- power LED on
	LED0 <= '1';
	-- transfer LED connected to ROM CS
	LED1 <= not DSSLOT_ROMCS;

	process (CLK)
	begin
		DSSLOT_IO <= (others => 'Z');
		DSCART_IO <= (others => 'Z');

		if (DSSLOT_RESET='1') then					-- if no reset

			if (DSSLOT_EEPCS='0') then				-- if DS EEPROM is selected,
				DSCART_IO(7) <= DSSLOT_IO(7);			-- connect the corresponding
				DSSLOT_IO(6) <= DSCART_IO(6);			-- data lines
			end if;

			if (dsslot_romcs_buf='0') then			-- if DS ROM is selected

				if (not is_command) then				-- if data is being transferred

					-- here we replace the 4 bytes in the header
					-- CRC will work only for the Metroid demo

					-- bytes are replaced at positions 54, 55, 350 and 351

					if (data_cnt = 54 and patch_en) then
						DSSLOT_IO <= "00000000";
					elsif (data_cnt = 55 and patch_en) then
						DSSLOT_IO <= "00001000";
					elsif (data_cnt = 350 and patch_en) then
						DSSLOT_IO <= "11110010";
					elsif (data_cnt = 351 and patch_en) then
						DSSLOT_IO <= "00000111";
					else
						DSSLOT_IO <= DSCART_IO;		-- Metroid data for all other positions
					end if;


				end if;

				if (is_command) then				-- if command is being transferred
						DSCART_IO <= DSSLOT_IO;		-- we route it to the Metroid cart
				end if;

			end if;

		end if;
	end process;

	-- this process keeps track of number of transfers
	-- first transfer (transfer_cnt == 0) is always the header
	process (dsslot_romcs_buf)
	begin
		if (DSSLOT_RESET='0') then
			transfer_cnt <= 0;
		elsif rising_edge(dsslot_romcs_buf) then
			transfer_cnt <= transfer_cnt + 1;
		end if;
	end process;

	-- this process keeps track of number of bytes transferred between DS and cart
	-- since first 8 bytes are command bytes, we also set the is_command variable
	process (dsslot_clk_buf)
	begin

		if (dsslot_romcs_buf='1') then

			cmd_cnt <= 0;
			is_command <= true;

		elsif (rising_edge(dsslot_clk_buf)) then

			if (cmd_cnt = 7) then
				is_command <= false;
				data_cnt <= 0;
			end if;

			if (is_command) then
				cmd_cnt <= cmd_cnt + 1;
			else
				data_cnt <= data_cnt + 1;
			end if;

		end if;

	end process;

end Behavioral;
